@@ -3,7 +3,7 @@ import click
 import os
 
 from mlflow.models import Model
-from mlflow.models.flavor_backend_registry import get_flavor_backend
+from mlflow.models.model import MLMODEL_FILE_NAME
 from mlflow.store.artifact.models_artifact_repo import ModelsArtifactRepository
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.utils import cli_args
@@ -51,33 +51,45 @@ def serve(model_uri, port, host, workers, no_conda=False, install_mlflow=False):
             "data": [[1, 2, 3], [4, 5, 6]]
         }'
     """
-    return _get_flavor_backend(model_uri,
-                               no_conda=no_conda,
-                               workers=workers,
-                               install_mlflow=install_mlflow).serve(model_uri=model_uri, port=port,
-                                                                    host=host)
+    return _get_flavor_backend(
+        model_uri, no_conda=no_conda, workers=workers, install_mlflow=install_mlflow
+    ).serve(model_uri=model_uri, port=port, host=host)
 
 
 @commands.command("predict")
 @cli_args.MODEL_URI
-@click.option("--input-path", "-i", default=None,
-              help="CSV containing pandas DataFrame to predict against.")
-@click.option("--output-path", "-o", default=None,
-              help="File to output results to as json file. If not provided, output to stdout.")
-@click.option("--content-type", "-t", default="json",
-              help="Content type of the input file. Can be one of {'json', 'csv'}.")
-@click.option("--json-format", "-j", default="split",
-              help="Only applies if the content type is 'json'. Specify how the data is encoded.  "
-                   "Can be one of {'split', 'records'} mirroring the behavior of Pandas orient "
-                   "attribute. The default is 'split' which expects dict like data: "
-                   "{'index' -> [index], 'columns' -> [columns], 'data' -> [values]}, "
-                   "where index  is optional. For more information see "
-                   "https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_json"
-                   ".html")
+@click.option(
+    "--input-path", "-i", default=None, help="CSV containing pandas DataFrame to predict against."
+)
+@click.option(
+    "--output-path",
+    "-o",
+    default=None,
+    help="File to output results to as json file. If not provided, output to stdout.",
+)
+@click.option(
+    "--content-type",
+    "-t",
+    default="json",
+    help="Content type of the input file. Can be one of {'json', 'csv'}.",
+)
+@click.option(
+    "--json-format",
+    "-j",
+    default="split",
+    help="Only applies if the content type is 'json'. Specify how the data is encoded.  "
+    "Can be one of {'split', 'records'} mirroring the behavior of Pandas orient "
+    "attribute. The default is 'split' which expects dict like data: "
+    "{'index' -> [index], 'columns' -> [columns], 'data' -> [values]}, "
+    "where index  is optional. For more information see "
+    "https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_json"
+    ".html",
+)
 @cli_args.NO_CONDA
 @cli_args.INSTALL_MLFLOW
-def predict(model_uri, input_path, output_path, content_type, json_format, no_conda,
-            install_mlflow):
+def predict(
+    model_uri, input_path, output_path, content_type, json_format, no_conda, install_mlflow
+):
     """
     Generate predictions in json format using a saved MLflow model. For information about the input
     data formats accepted by this function, see the following documentation:
@@ -85,12 +97,13 @@ def predict(model_uri, input_path, output_path, content_type, json_format, no_co
     """
     if content_type == "json" and json_format not in ("split", "records"):
         raise Exception("Unsupported json format '{}'.".format(json_format))
-    return _get_flavor_backend(model_uri, no_conda=no_conda,
-                               install_mlflow=install_mlflow).predict(model_uri=model_uri,
-                                                                      input_path=input_path,
-                                                                      output_path=output_path,
-                                                                      content_type=content_type,
-                                                                      json_format=json_format)
+    return _get_flavor_backend(model_uri, no_conda=no_conda, install_mlflow=install_mlflow).predict(
+        model_uri=model_uri,
+        input_path=input_path,
+        output_path=output_path,
+        content_type=content_type,
+        json_format=json_format,
+    )
 
 
 @commands.command("prepare-env")
@@ -105,14 +118,14 @@ def prepare_env(model_uri, no_conda, install_mlflow):
 
     This method is experimental and may be removed in a future release without warning.
     """
-    return _get_flavor_backend(model_uri, no_conda=no_conda,
-                               install_mlflow=install_mlflow).prepare_env(model_uri=model_uri)
+    return _get_flavor_backend(
+        model_uri, no_conda=no_conda, install_mlflow=install_mlflow
+    ).prepare_env(model_uri=model_uri)
 
 
 @commands.command("build-docker")
 @cli_args.MODEL_URI
-@click.option("--name", "-n", default="mlflow-pyfunc-servable",
-              help="Name to use for built image")
+@click.option("--name", "-n", default="mlflow-pyfunc-servable", help="Name to use for built image")
 @cli_args.INSTALL_MLFLOW
 def build_docker(model_uri, name, install_mlflow):
     """
@@ -149,19 +162,22 @@ def build_docker(model_uri, name, install_mlflow):
     same.
     """
     mlflow_home = os.environ.get("MLFLOW_HOME", None)
-    _get_flavor_backend(model_uri, docker_build=True).build_image(model_uri, name,
-                                                                  mlflow_home=mlflow_home,
-                                                                  install_mlflow=install_mlflow)
+    _get_flavor_backend(model_uri, docker_build=True).build_image(
+        model_uri, name, mlflow_home=mlflow_home, install_mlflow=install_mlflow
+    )
 
 
 def _get_flavor_backend(model_uri, **kwargs):
+    from mlflow.models.flavor_backend_registry import get_flavor_backend
+
     with TempDir() as tmp:
         if ModelsArtifactRepository.is_models_uri(model_uri):
             underlying_model_uri = ModelsArtifactRepository.get_underlying_uri(model_uri)
         else:
             underlying_model_uri = model_uri
         local_path = _download_artifact_from_uri(
-            append_to_uri_path(underlying_model_uri, "MLmodel"), output_path=tmp.path())
+            append_to_uri_path(underlying_model_uri, MLMODEL_FILE_NAME), output_path=tmp.path()
+        )
         model = Model.load(local_path)
     flavor_name, flavor_backend = get_flavor_backend(model, **kwargs)
     if flavor_backend is None:
