@@ -8,8 +8,9 @@ import atexit
 import time
 import logging
 import inspect
+from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING
 
-from mlflow.entities import Run, RunStatus, Param, RunTag, Metric, ViewType
+from mlflow.entities import Experiment, Run, RunInfo, RunStatus, Param, RunTag, Metric, ViewType
 from mlflow.entities.lifecycle_stage import LifecycleStage
 from mlflow.exceptions import MlflowException
 from mlflow.tracking.client import MlflowClient
@@ -18,7 +19,7 @@ from mlflow.tracking.context import registry as context_registry
 from mlflow.store.tracking import SEARCH_MAX_RESULTS_DEFAULT
 from mlflow.utils import env
 from mlflow.utils.autologging_utils import (
-    _is_testing,
+    is_testing,
     autologging_integration,
     AUTOLOGGING_INTEGRATIONS,
     autologging_is_disabled,
@@ -29,6 +30,14 @@ from mlflow.utils.mlflow_tags import MLFLOW_PARENT_RUN_ID, MLFLOW_RUN_NAME
 from mlflow.utils.validation import _validate_run_id
 from mlflow.utils.annotations import experimental
 
+if TYPE_CHECKING:
+    import pandas  # pylint: disable=unused-import
+    import matplotlib  # pylint: disable=unused-import
+    import plotly  # pylint: disable=unused-import
+    import numpy  # pylint: disable=unused-import
+    import PIL  # pylint: disable=unused-import
+
+
 _EXPERIMENT_ID_ENV_VAR = "MLFLOW_EXPERIMENT_ID"
 _EXPERIMENT_NAME_ENV_VAR = "MLFLOW_EXPERIMENT_NAME"
 _RUN_ID_ENV_VAR = "MLFLOW_RUN_ID"
@@ -41,7 +50,7 @@ NUM_RUNS_PER_PAGE_PANDAS = 10000
 _logger = logging.getLogger(__name__)
 
 
-def set_experiment(experiment_name):
+def set_experiment(experiment_name: str) -> None:
     """
     Set given experiment as active experiment. If experiment does not exist, create an experiment
     with provided name.
@@ -102,7 +111,13 @@ class ActiveRun(Run):  # pylint: disable=W0223
         return exc_type is None
 
 
-def start_run(run_id=None, experiment_id=None, run_name=None, nested=False, tags=None):
+def start_run(
+    run_id: str = None,
+    experiment_id: Optional[str] = None,
+    run_name: Optional[str] = None,
+    nested: bool = False,
+    tags: Optional[Dict[str, Any]] = None,
+) -> ActiveRun:
     """
     Start a new MLflow run, setting it as the active run under which metrics and parameters
     will be logged. The return value can be used as a context manager within a ``with`` block;
@@ -231,7 +246,7 @@ def start_run(run_id=None, experiment_id=None, run_name=None, nested=False, tags
     return _active_run_stack[-1]
 
 
-def end_run(status=RunStatus.to_string(RunStatus.FINISHED)):
+def end_run(status: str = RunStatus.to_string(RunStatus.FINISHED)) -> None:
     """End an active MLflow run (if there is one).
 
     .. code-block:: python
@@ -272,7 +287,7 @@ def end_run(status=RunStatus.to_string(RunStatus.FINISHED)):
 atexit.register(end_run)
 
 
-def active_run():
+def active_run() -> Optional[ActiveRun]:
     """Get the currently active ``Run``, or None if no such run exists.
 
     **Note**: You cannot access currently-active run attributes
@@ -297,7 +312,7 @@ def active_run():
     return _active_run_stack[-1] if len(_active_run_stack) > 0 else None
 
 
-def get_run(run_id):
+def get_run(run_id: str) -> Run:
     """
     Fetch the run from backend store. The resulting :py:class:`Run <mlflow.entities.Run>`
     contains a collection of run metadata -- :py:class:`RunInfo <mlflow.entities.RunInfo>`,
@@ -331,7 +346,7 @@ def get_run(run_id):
     return MlflowClient().get_run(run_id)
 
 
-def log_param(key, value):
+def log_param(key: str, value: Any) -> None:
     """
     Log a parameter under the current run. If no run is active, this method will create
     a new active run.
@@ -351,7 +366,7 @@ def log_param(key, value):
     MlflowClient().log_param(run_id, key, value)
 
 
-def set_tag(key, value):
+def set_tag(key: str, value: Any) -> None:
     """
     Set a tag under the current run. If no run is active, this method will create a
     new active run.
@@ -371,7 +386,7 @@ def set_tag(key, value):
     MlflowClient().set_tag(run_id, key, value)
 
 
-def delete_tag(key):
+def delete_tag(key: str) -> None:
     """
     Delete a tag from a run. This is irreversible. If no run is active, this method
     will create a new active run.
@@ -396,7 +411,7 @@ def delete_tag(key):
     MlflowClient().delete_tag(run_id, key)
 
 
-def log_metric(key, value, step=None):
+def log_metric(key: str, value: float, step: Optional[int] = None) -> None:
     """
     Log a metric under the current run. If no run is active, this method will create
     a new active run.
@@ -419,7 +434,7 @@ def log_metric(key, value, step=None):
     MlflowClient().log_metric(run_id, key, value, int(time.time() * 1000), step or 0)
 
 
-def log_metrics(metrics, step=None):
+def log_metrics(metrics: Dict[str, float], step: Optional[int] = None) -> None:
     """
     Log multiple metrics for the current run. If no run is active, this method will create a new
     active run.
@@ -450,7 +465,7 @@ def log_metrics(metrics, step=None):
     MlflowClient().log_batch(run_id=run_id, metrics=metrics_arr, params=[], tags=[])
 
 
-def log_params(params):
+def log_params(params: Dict[str, Any]) -> None:
     """
     Log a batch of params for the current run. If no run is active, this method will create a
     new active run.
@@ -475,7 +490,7 @@ def log_params(params):
     MlflowClient().log_batch(run_id=run_id, metrics=[], params=params_arr, tags=[])
 
 
-def set_tags(tags):
+def set_tags(tags: Dict[str, Any]) -> None:
     """
     Log a batch of tags for the current run. If no run is active, this method will create a
     new active run.
@@ -502,7 +517,7 @@ def set_tags(tags):
     MlflowClient().log_batch(run_id=run_id, metrics=[], params=[], tags=tags_arr)
 
 
-def log_artifact(local_path, artifact_path=None):
+def log_artifact(local_path: str, artifact_path: Optional[str] = None) -> None:
     """
     Log a local file or directory as an artifact of the currently active run. If no run is
     active, this method will create a new active run.
@@ -529,7 +544,7 @@ def log_artifact(local_path, artifact_path=None):
     MlflowClient().log_artifact(run_id, local_path, artifact_path)
 
 
-def log_artifacts(local_dir, artifact_path=None):
+def log_artifacts(local_dir: str, artifact_path: Optional[str] = None) -> None:
     """
     Log all the contents of a local directory as artifacts of the run. If no run is active,
     this method will create a new active run.
@@ -562,7 +577,7 @@ def log_artifacts(local_dir, artifact_path=None):
     MlflowClient().log_artifacts(run_id, local_dir, artifact_path)
 
 
-def log_text(text, artifact_file):
+def log_text(text: str, artifact_file: str) -> None:
     """
     Log text as an artifact.
 
@@ -590,15 +605,16 @@ def log_text(text, artifact_file):
 
 
 @experimental
-def log_dict(dictionary, artifact_file):
+def log_dict(dictionary: Any, artifact_file: str) -> None:
     """
-    Log a dictionary as an artifact. The serialization format (JSON or YAML) is automatically
-    inferred from the extension of `artifact_file`. If the file extension doesn't exist or match
-    any of [".json", ".yml", ".yaml"], JSON format is used.
+    Log a JSON/YAML-serializable object (e.g. `dict`) as an artifact. The serialization
+    format (JSON or YAML) is automatically inferred from the extension of `artifact_file`.
+    If the file extension doesn't exist or match any of [".json", ".yml", ".yaml"],
+    JSON format is used.
 
     :param dictionary: Dictionary to log.
     :param artifact_file: The run-relative artifact file path in posixpath format to which
-                            the dictionary is saved (e.g. "dir/data.json").
+                          the dictionary is saved (e.g. "dir/data.json").
 
     .. code-block:: python
         :caption: Example
@@ -624,7 +640,9 @@ def log_dict(dictionary, artifact_file):
 
 
 @experimental
-def log_figure(figure, artifact_file):
+def log_figure(
+    figure: Union["matplotlib.figure.Figure", "plotly.graph_objects.Figure"], artifact_file: str
+) -> None:
     """
     Log a figure as an artifact. The following figure objects are supported:
 
@@ -669,7 +687,7 @@ def log_figure(figure, artifact_file):
 
 
 @experimental
-def log_image(image, artifact_file):
+def log_image(image: Union["numpy.ndarray", "PIL.Image.Image"], artifact_file: str) -> None:
     """
     Log an image as an artifact. The following image objects are supported:
 
@@ -737,7 +755,7 @@ def _record_logged_model(mlflow_model):
     MlflowClient()._record_logged_model(run_id, mlflow_model)
 
 
-def get_experiment(experiment_id):
+def get_experiment(experiment_id: str) -> Experiment:
     """
     Retrieve an experiment by experiment_id from the backend store
 
@@ -766,12 +784,13 @@ def get_experiment(experiment_id):
     return MlflowClient().get_experiment(experiment_id)
 
 
-def get_experiment_by_name(name):
+def get_experiment_by_name(name: str) -> Optional[Experiment]:
     """
     Retrieve an experiment by experiment name from the backend store
 
     :param name: The case senstive experiment name.
-    :return: :py:class:`mlflow.entities.Experiment`
+    :return: An instance of :py:class:`mlflow.entities.Experiment`
+             if an experiment with the specified name exists, otherwise None.
 
     .. code-block:: python
         :caption: Example
@@ -796,7 +815,7 @@ def get_experiment_by_name(name):
     return MlflowClient().get_experiment_by_name(name)
 
 
-def create_experiment(name, artifact_location=None):
+def create_experiment(name: str, artifact_location: Optional[str] = None) -> str:
     """
     Create an experiment.
 
@@ -831,7 +850,7 @@ def create_experiment(name, artifact_location=None):
     return MlflowClient().create_experiment(name, artifact_location)
 
 
-def delete_experiment(experiment_id):
+def delete_experiment(experiment_id: str) -> None:
     """
     Delete an experiment from the backend store.
 
@@ -861,7 +880,7 @@ def delete_experiment(experiment_id):
     MlflowClient().delete_experiment(experiment_id)
 
 
-def delete_run(run_id):
+def delete_run(run_id: str) -> None:
     """
     Deletes a run with the given ID.
 
@@ -889,7 +908,7 @@ def delete_run(run_id):
     MlflowClient().delete_run(run_id)
 
 
-def get_artifact_uri(artifact_path=None):
+def get_artifact_uri(artifact_path: Optional[str] = None) -> str:
     """
     Get the absolute URI of the specified artifact in the currently active run.
     If `path` is not specified, the artifact root URI of the currently active
@@ -941,13 +960,13 @@ def get_artifact_uri(artifact_path=None):
 
 
 def search_runs(
-    experiment_ids=None,
-    filter_string="",
-    run_view_type=ViewType.ACTIVE_ONLY,
-    max_results=SEARCH_MAX_RESULTS_PANDAS,
-    order_by=None,
-    output_format="pandas",
-):
+    experiment_ids: Optional[List[str]] = None,
+    filter_string: str = "",
+    run_view_type: int = ViewType.ACTIVE_ONLY,
+    max_results: int = SEARCH_MAX_RESULTS_PANDAS,
+    order_by: Optional[List[str]] = None,
+    output_format: str = "pandas",
+) -> Union[List[Run], "pandas.DataFrame"]:
     """
     Get a pandas DataFrame of runs that fit the search criteria.
 
@@ -1095,11 +1114,11 @@ def search_runs(
 
 
 def list_run_infos(
-    experiment_id,
-    run_view_type=ViewType.ACTIVE_ONLY,
-    max_results=SEARCH_MAX_RESULTS_DEFAULT,
-    order_by=None,
-):
+    experiment_id: str,
+    run_view_type: int = ViewType.ACTIVE_ONLY,
+    max_results: int = SEARCH_MAX_RESULTS_DEFAULT,
+    order_by: Optional[List[str]] = None,
+) -> List[RunInfo]:
     """
     Return run information for runs which belong to the experiment_id.
 
@@ -1221,14 +1240,15 @@ def _get_experiment_id():
 
 @autologging_integration("mlflow")
 def autolog(
-    log_input_examples=False,
-    log_model_signatures=True,
-    log_models=True,
-    disable=False,
-    exclusive=False,
-    disable_for_unsupported_versions=False,
-    silent=False,
-):  # pylint: disable=unused-argument
+    log_input_examples: bool = False,
+    log_model_signatures: bool = True,
+    log_models: bool = True,
+    disable: bool = False,
+    exclusive: bool = False,
+    disable_for_unsupported_versions: bool = False,
+    silent: bool = False,
+    # pylint: disable=unused-argument
+) -> None:
     """
     Enables (or disables) and configures autologging for all supported integrations.
 
@@ -1340,6 +1360,7 @@ def autolog(
         gluon,
         xgboost,
         lightgbm,
+        pyspark,
         statsmodels,
         spark,
         sklearn,
@@ -1361,6 +1382,7 @@ def autolog(
         "sklearn": sklearn.autolog,
         "fastai": fastai.autolog,
         "pyspark": spark.autolog,
+        "pyspark.ml": pyspark.ml.autolog,
         # TODO: Broaden this beyond pytorch_lightning as we add autologging support for more
         # Pytorch frameworks under mlflow.pytorch.autolog
         "pytorch_lightning": pytorch.autolog,
@@ -1404,7 +1426,7 @@ def autolog(
             ) and not autologging_params.get("silent", False):
                 _logger.info("Autologging successfully enabled for %s.", module.__name__)
         except Exception as e:
-            if _is_testing():
+            if is_testing():
                 # Raise unexpected exceptions in test mode in order to detect
                 # errors within dependent autologging integrations
                 raise
@@ -1418,22 +1440,23 @@ def autolog(
     # for each autolog library (except pyspark), register a post-import hook.
     # this way, we do not send any errors to the user until we know they are using the library.
     # the post-import hook also retroactively activates for previously-imported libraries.
-    for module in list(set(LIBRARY_TO_AUTOLOG_FN.keys()) - set(["pyspark"])):
+    for module in list(set(LIBRARY_TO_AUTOLOG_FN.keys()) - set(["pyspark", "pyspark.ml"])):
         register_post_import_hook(setup_autologging, module, overwrite=True)
 
     # for pyspark, we activate autologging immediately, without waiting for a module import.
     # this is because on Databricks a SparkSession already exists and the user can directly
     #   interact with it, and this activity should be logged.
     try:
-        autologging_params = get_autologging_params(spark.autolog)
-        spark.autolog(**autologging_params)
+        spark.autolog(**get_autologging_params(spark.autolog))
+        pyspark.ml.autolog(**get_autologging_params(pyspark.ml.autolog))
     except ImportError as ie:
         # if pyspark isn't installed, a user could potentially install it in the middle
         #   of their session so we want to enable autologging once they do
         if "pyspark" in str(ie):
             register_post_import_hook(setup_autologging, "pyspark", overwrite=True)
+            register_post_import_hook(setup_autologging, "pyspark.ml", overwrite=True)
     except Exception as e:
-        if _is_testing():
+        if is_testing():
             # Raise unexpected exceptions in test mode in order to detect
             # errors within dependent autologging integrations
             raise
