@@ -34,7 +34,7 @@ MAX_ENTITY_KEY_LENGTH = 250
 MAX_MODEL_REGISTRY_TAG_KEY_LENGTH = 250
 MAX_MODEL_REGISTRY_TAG_VALUE_LENGTH = 5000
 
-_UNSUPPORTED_DB_TYPE_MSG = "Supported database engines are {%s}" % ', '.join(DATABASE_ENGINES)
+_UNSUPPORTED_DB_TYPE_MSG = "Supported database engines are {%s}" % ", ".join(DATABASE_ENGINES)
 
 
 def bad_path_message(name):
@@ -46,17 +46,30 @@ def bad_path_message(name):
 
 def path_not_unique(name):
     norm = posixpath.normpath(name)
-    return norm != name or norm == '.' or norm.startswith('..') or norm.startswith('/')
+    return norm != name or norm == "." or norm.startswith("..") or norm.startswith("/")
 
 
 def _validate_metric_name(name):
     """Check that `name` is a valid metric name and raise an exception if it isn't."""
     if name is None or not _VALID_PARAM_AND_METRIC_NAMES.match(name):
-        raise MlflowException("Invalid metric name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid metric name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE),
+            INVALID_PARAMETER_VALUE,
+        )
     if path_not_unique(name):
-        raise MlflowException("Invalid metric name: '%s'. %s" % (name, bad_path_message(name)),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid metric name: '%s'. %s" % (name, bad_path_message(name)),
+            INVALID_PARAMETER_VALUE,
+        )
+
+
+def _is_numeric(value):
+    """
+    Returns True if the passed-in value is numeric.
+    """
+    # Note that `isinstance(bool_value, numbers.Number)` returns `True` because `bool` is a
+    # subclass of `int`.
+    return not isinstance(value, bool) and isinstance(value, numbers.Number)
 
 
 def _validate_metric(key, value, timestamp, step):
@@ -65,23 +78,28 @@ def _validate_metric(key, value, timestamp, step):
     it isn't.
     """
     _validate_metric_name(key)
-    if not isinstance(value, numbers.Number):
+    # value must be a Number
+    # since bool is an instance of Number check for bool additionally
+    if not _is_numeric(value):
         raise MlflowException(
             "Got invalid value %s for metric '%s' (timestamp=%s). Please specify value as a valid "
             "double (64-bit floating point)" % (value, key, timestamp),
-            INVALID_PARAMETER_VALUE)
+            INVALID_PARAMETER_VALUE,
+        )
 
     if not isinstance(timestamp, numbers.Number) or timestamp < 0:
         raise MlflowException(
             "Got invalid timestamp %s for metric '%s' (value=%s). Timestamp must be a nonnegative "
             "long (64-bit integer) " % (timestamp, key, value),
-            INVALID_PARAMETER_VALUE)
+            INVALID_PARAMETER_VALUE,
+        )
 
     if not isinstance(step, numbers.Number):
         raise MlflowException(
             "Got invalid step %s for metric '%s' (value=%s). Step must be a valid long "
             "(64-bit integer)." % (step, key, value),
-            INVALID_PARAMETER_VALUE)
+            INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_param(key, value):
@@ -133,29 +151,37 @@ def _validate_model_version_tag(key, value):
 def _validate_param_name(name):
     """Check that `name` is a valid parameter name and raise an exception if it isn't."""
     if name is None or not _VALID_PARAM_AND_METRIC_NAMES.match(name):
-        raise MlflowException("Invalid parameter name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid parameter name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE),
+            INVALID_PARAMETER_VALUE,
+        )
     if path_not_unique(name):
-        raise MlflowException("Invalid parameter name: '%s'. %s" % (name, bad_path_message(name)),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid parameter name: '%s'. %s" % (name, bad_path_message(name)),
+            INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_tag_name(name):
     """Check that `name` is a valid tag name and raise an exception if it isn't."""
     # Reuse param & metric check.
     if name is None or not _VALID_PARAM_AND_METRIC_NAMES.match(name):
-        raise MlflowException("Invalid tag name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid tag name: '%s'. %s" % (name, _BAD_CHARACTERS_MESSAGE), INVALID_PARAMETER_VALUE
+        )
     if path_not_unique(name):
-        raise MlflowException("Invalid tag name: '%s'. %s" % (name, bad_path_message(name)),
-                              INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid tag name: '%s'. %s" % (name, bad_path_message(name)), INVALID_PARAMETER_VALUE
+        )
 
 
 def _validate_length_limit(entity_name, limit, value):
     if len(value) > limit:
         raise MlflowException(
-            "%s '%s' had length %s, which exceeded length limit of %s" %
-            (entity_name, value[:250], len(value), limit), error_code=INVALID_PARAMETER_VALUE)
+            "%s '%s' had length %s, which exceeded length limit of %s"
+            % (entity_name, value[:250], len(value), limit),
+            error_code=INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_run_id(run_id):
@@ -167,15 +193,18 @@ def _validate_run_id(run_id):
 def _validate_experiment_id(exp_id):
     """Check that `experiment_id`is a valid string or None, raise an exception if it isn't."""
     if exp_id is not None and _EXPERIMENT_ID_REGEX.match(exp_id) is None:
-        raise MlflowException("Invalid experiment ID: '%s'" % exp_id,
-                              error_code=INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid experiment ID: '%s'" % exp_id, error_code=INVALID_PARAMETER_VALUE
+        )
 
 
 def _validate_batch_limit(entity_name, limit, length):
     if length > limit:
-        error_msg = ("A batch logging request can contain at most {limit} {name}. "
-                     "Got {count} {name}. Please split up {name} across multiple requests and try "
-                     "again.").format(name=entity_name, count=length, limit=limit)
+        error_msg = (
+            "A batch logging request can contain at most {limit} {name}. "
+            "Got {count} {name}. Please split up {name} across multiple requests and try "
+            "again."
+        ).format(name=entity_name, count=length, limit=limit)
         raise MlflowException(error_msg, error_code=INVALID_PARAMETER_VALUE)
 
 
@@ -185,8 +214,9 @@ def _validate_batch_log_limits(metrics, params, tags):
     _validate_batch_limit(entity_name="params", limit=MAX_PARAMS_TAGS_PER_BATCH, length=len(params))
     _validate_batch_limit(entity_name="tags", limit=MAX_PARAMS_TAGS_PER_BATCH, length=len(tags))
     total_length = len(metrics) + len(params) + len(tags)
-    _validate_batch_limit(entity_name="metrics, params, and tags",
-                          limit=MAX_ENTITIES_PER_BATCH, length=total_length)
+    _validate_batch_limit(
+        entity_name="metrics, params, and tags", limit=MAX_ENTITIES_PER_BATCH, length=total_length
+    )
 
 
 def _validate_batch_log_data(metrics, params, tags):
@@ -206,41 +236,48 @@ def _validate_batch_log_data(metrics, params, tags):
 
 def _validate_batch_log_api_req(json_req):
     if len(json_req) > MAX_BATCH_LOG_REQUEST_SIZE:
-        error_msg = ("Batched logging API requests must be at most {limit} bytes, got a "
-                     "request of size {size}.").format(
-            limit=MAX_BATCH_LOG_REQUEST_SIZE, size=len(json_req))
+        error_msg = (
+            "Batched logging API requests must be at most {limit} bytes, got a "
+            "request of size {size}."
+        ).format(limit=MAX_BATCH_LOG_REQUEST_SIZE, size=len(json_req))
         raise MlflowException(error_msg, error_code=INVALID_PARAMETER_VALUE)
 
 
 def _validate_experiment_name(experiment_name):
     """Check that `experiment_name` is a valid string and raise an exception if it isn't."""
     if experiment_name == "" or experiment_name is None:
-        raise MlflowException("Invalid experiment name: '%s'" % experiment_name,
-                              error_code=INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid experiment name: '%s'" % experiment_name, error_code=INVALID_PARAMETER_VALUE
+        )
 
     if not is_string_type(experiment_name):
-        raise MlflowException("Invalid experiment name: %s. Expects a string." % experiment_name,
-                              error_code=INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Invalid experiment name: %s. Expects a string." % experiment_name,
+            error_code=INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_model_name(model_name):
     if model_name is None or model_name == "":
-        raise MlflowException('Registered model name cannot be empty.', INVALID_PARAMETER_VALUE)
+        raise MlflowException("Registered model name cannot be empty.", INVALID_PARAMETER_VALUE)
 
 
 def _validate_model_version(model_version):
     try:
         model_version = int(model_version)
     except ValueError:
-        raise MlflowException("Model version must be an integer, got '{}'"
-                              .format(model_version), error_code=INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Model version must be an integer, got '{}'".format(model_version),
+            error_code=INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_experiment_artifact_location(artifact_location):
     if artifact_location is not None and artifact_location.startswith("runs:"):
-        raise MlflowException("Artifact location cannot be a runs:/ URI. Given: '%s'"
-                              % artifact_location,
-                              error_code=INVALID_PARAMETER_VALUE)
+        raise MlflowException(
+            "Artifact location cannot be a runs:/ URI. Given: '%s'" % artifact_location,
+            error_code=INVALID_PARAMETER_VALUE,
+        )
 
 
 def _validate_db_type_string(db_type):
